#include "mainwindow.h"
#include "setnewmap.h"
#include "settings.h"
#include "goalsettings.h"
#include <QAction>
#include <QDebug>
#include <QMenu>
#include <QFileDialog>
#include <QMessageBox>
#include <fstream>
MainWindow::MainWindow(QWidget *parent)
    : QMainWindow(parent)
{

    QAction* actOpenFile = new QAction("&Open...",this);
    QAction* actSaveMap = new QAction("&Save...",this);
    QAction* actQuit = new QAction("&Quit",this);

    QAction* actSettings = new QAction("&View settings...",this);
    actSetBarrier = new QAction("Set barrier",this);
    actSetBarrier->setCheckable(true);
    actSetBarrier->setChecked(false);
    actSetGoal = new QAction("Set goal",this);
    actSetGoal->setCheckable(true);
    actSetGoal->setChecked(false);
    QAction* actNewMap = new QAction("&New map...",this);


    QAction* actPlan = new QAction("&Plan strategy",this);

    actViewVal = new QAction("View values",this);
    actViewVal->setCheckable(true);
    actViewVal->setChecked(true);

    actViewArr = new QAction("View arrows",this);
    actViewArr->setCheckable(true);
    actViewArr->setChecked(true);


    toolBar = addToolBar("Main tool bar");
    toolBar->addAction(actOpenFile);
    toolBar->addAction(actSaveMap);
    toolBar->addAction(actQuit);
    toolBar->addSeparator();
    toolBar->addAction(actSettings);
    toolBar->addAction(actSetBarrier);
    toolBar->addAction(actSetGoal);
    toolBar->addAction(actNewMap);
    toolBar->addSeparator();
    toolBar->addAction(actPlan);


    menuFile = new QMenu("&File", this);
    menuFile->addAction(actOpenFile);
    menuFile->addAction(actSaveMap);
    menuFile->addAction(actQuit);
    menuBar()->addMenu(menuFile);

    menuSet = new QMenu("&Settings", this);
    menuSet->addAction(actNewMap);
    menuSet->addAction(actSettings);
    menuSet->addAction(actSetBarrier);
    menuSet->addAction(actSetGoal);
    menuBar()->addMenu(menuSet);

    menuRun = new QMenu("P&rocess",this);
    menuRun->addAction(actPlan);
    menuBar()->addMenu(menuRun);

    menuView = new QMenu("View", this);
    menuView->addAction(actViewVal);
    menuView->addAction(actViewArr);
    menuBar()->addMenu(menuView);

    //this->setMenuBar(menu);
    mv = new MapView(this);

    setCentralWidget(mv);

    setWindowTitle("Strategizer");
    connect(actQuit, SIGNAL(triggered()),
            this, SLOT(close()));

    connect(actNewMap, SIGNAL(triggered()),
            this, SLOT(newMap()));

    connect(actOpenFile, SIGNAL(triggered()),
            this, SLOT(loadFile()));

    connect(actPlan, SIGNAL(triggered()),
            this, SLOT(planStrategy()));

    connect(actSaveMap, SIGNAL(triggered()),
            this, SLOT(saveSettings()));



    connect(actSettings, SIGNAL(triggered()),
            this, SLOT(showSettings()));

    connect(actViewVal, SIGNAL(toggled(bool)),
            this, SLOT(valToggled(bool)));

    connect(actViewArr, SIGNAL(toggled(bool)),
            this, SLOT(arrToggled(bool)));

    connect(actSetBarrier, SIGNAL(toggled(bool)),
            this, SLOT(barrToggled(bool)));

    connect(actSetGoal, SIGNAL(toggled(bool)),
            this, SLOT(goalToggled(bool)));

    connect(mv, SIGNAL(fieldClicked(int, int)),
            this, SLOT(setFieldProperty(int,int)));

    initValues();

}

MainWindow::~MainWindow()
{

}

void MainWindow::newMap()
{

    SetNewMap dialog;
    dialog.setNumberOfCols(mainMap.getCols());
    dialog.setNumberOfRows(mainMap.getRows());

    if(dialog.exec() == QDialog::Accepted){


        mainMap.alocateMap(dialog.getNumberOfRows(), dialog.getNumberOfCols());

        mv->updateView(mainMap);
        if (this->windowState() != Qt::WindowMaximized){
            resize(mv->size());
        }
    }
}

void MainWindow::loadFile()
{
    QString fileName = QFileDialog::getOpenFileName(this, tr("Select map file"),
                                                    QDir::homePath(),
                                                    tr("Config file (*.smap *.txt);;All files (*)"));
    if (!fileName.isEmpty()) {
        qDebug()<<fileName;
        mainMap.loadFromFile(fileName.toLocal8Bit().data());
        strat.loadFromFile(fileName.toLocal8Bit().data());
        showMap(&mainMap);
        if (this->windowState() != Qt::WindowMaximized){
            //resize(mv->size());
        }
    }

}

void MainWindow::planStrategy()
{
    qDebug() << strat.getPOK();
    resMap.tunel(strat.plan(mainMap));
    showMap(&resMap);
    if (this->windowState() != Qt::WindowMaximized){
        resize(mv->size());
    }
}

void MainWindow::showSettings()
{
    qDebug() << "oteviram dialog";
    Settings dialog;

    dialog.setPStraight(strat.getPOK());
    dialog.setPRight(strat.getPRight());
    dialog.setPLeft(strat.getPLeft());
    dialog.setPBack(strat.getPBack());

    dialog.setMaxN(strat.getNMax());
    dialog.setCost(strat.getStepCost());
    dialog.setGamma(strat.getGamma());

    if(dialog.exec() == QDialog::Accepted){
        qDebug() << "ukladam nastaveni";

        strat.setPOK(dialog.getPStraight());
        strat.setPRight(dialog.getPRight());
        strat.setPLeft(dialog.getPLeft());
        strat.setPBack(dialog.getPBack());
        strat.setGamma(dialog.getGamma());
        strat.setNMax(dialog.getMaxN());
        strat.setStepCost(dialog.getCost());

        if (fabs(strat.getPOK() + strat.getPRight() + strat.getPLeft() + strat.getPBack() - 1.0) > 1e-5) {

            QMessageBox::warning(this, "wrong probability setting", "The sum of propabilities must be equal to 100%");
        }

    }
}




void MainWindow::saveSettings()
{
    QString fileName = QFileDialog::getSaveFileName(this, tr("Save map"),
                                                    QDir::homePath(),
                                                    tr("Config file (*.smap *.txt);;All files (*)"));
    if (!fileName.isEmpty()) {
        qDebug()<<fileName;
        std::ofstream saveFile(fileName.toLocal8Bit().data());

        saveFile << "# map information\n\n";

        saveFile << "dimensions " << mainMap.getRows() << " " << mainMap.getCols() << std::endl;

        saveFile << "\nbarriers" << std::endl;

        for (int i = 0; i < mainMap.getRows(); i++) {
            for (int j = 0; j < mainMap.getCols(); j++) {
                if (mainMap.at(i,j).getFlag() == Field::barier) {
                    saveFile << i << " " << j << "\n";
                }
            }
        }

        saveFile << "\ngoals\n";

        for (int i = 0; i < mainMap.getRows(); i++) {
            for (int j = 0; j < mainMap.getCols(); j++) {
                qDebug() << i << j << mainMap.at(i,j).getFlag();
                if (mainMap.at(i,j).getFlag() == Field::goal) {
                    saveFile << i << " " << j  << " " << mainMap.at(i,j).getValue() << "\n";
                }
            }
        }

        saveFile << std::endl;

        saveFile << "# strategist info\n";

        saveFile << "p_straight " << strat.getPOK() << std::endl;

        saveFile << "p_right " << strat.getPRight() << std::endl;

        saveFile << "p_left " << strat.getPLeft() << std::endl;

        saveFile << "p_back " << strat.getPOK() << std::endl;

        saveFile << "maxn " << strat.getNMax() << std::endl;

        saveFile << "gamma " << strat.getGamma() << std::endl;

        saveFile << "step_cost " << strat.getStepCost() << std::endl;


    }

}






void MainWindow::showMap(Map* map)
{
    if (map) {

        showedMap = map;

        mv->updateView(*map);
    }
}




void MainWindow::initValues()
{
    mv->setviewMode(3);
}





void MainWindow::valToggled(bool checked)
{
    int view = mv->getviewMode();
    if (checked) {
        view |= 1;
    } else {
        view = ~view;
        view |= 1;
        view = ~view;
    }
    mv->setviewMode(view);
    showMap(showedMap);
}



void MainWindow::arrToggled(bool checked)
{
    int view = mv->getviewMode();
    if (checked) {
        view |= 2;
    } else {
        view = ~view;
        view |= 2;
        view = ~view;
    }
    mv->setviewMode(view);
    showMap(showedMap);
}

void MainWindow::barrToggled(bool checked)
{
    if (checked) {
        actSetGoal->setChecked(false);
    }
}

void MainWindow::goalToggled(bool checked)
{
    if (checked) {
        actSetBarrier->setChecked(false);
    }
}

void MainWindow::setFieldProperty(int i, int j)
{
    if(i >= 0 && i < mainMap.getCols() && j >= 0 && j < mainMap.getRows()){
        if (actSetBarrier->isChecked()){
            if (mainMap.at(j,i).getFlag()== Field::barier){
                mainMap.at(j,i).setFlag(Field::free);
            } else {
                mainMap.at(j,i).setFlag(Field::barier);
            }
            mv->updateView(mainMap);
        }
        if (actSetGoal->isChecked()){
            GoalSettings g(this);
            if(mainMap.at(j,i).getFlag() == Field::goal){
                g.setValue(int(mainMap.at(j,i).getValue()));
            }else {
                g.setValue(0);
            }
             if(g.exec() == QDialog::Accepted){
                 if(g.removeGoal() && mainMap.at(j,i).getFlag() == Field::goal) {
                     mainMap.at(j,i).setFlag(Field::free);
                     mainMap.at(j,i).setValue(0);
                 }
                 if(!g.removeGoal()){
                     mainMap.at(j,i).setFlag(Field::goal);
                     mainMap.at(j,i).setValue(g.getValue());
                 }
                 mv->updateView(mainMap);
             }
        }
    }
}
