#include "map.h"
#include <string>
#include <sstream>
#include <iomanip>
Map::Map(const int rows, const int cols)
{
    if (rows != 0 || cols != 0) {
        alocateMap(rows, cols);
    } else {
        setRows(0);
        setCols(0);
    }
}

Map::Map(const char *fileName)
{
    loadFromFile(fileName);
}

Map::Map(Map &other)
{
    Cols = 0;
    Rows = 0;
    copy(other);

}

Map::Map(Map &&other)
{
    Cols = 0;
    Rows = 0;
    map = other.map;
    other.map = nullptr;
    other.setCols(0);
    other.setRows(0);

}

void Map::copy(Map &other)
{
    if (getCols() != other.getCols() || getRows() != other.getRows()){

        deleteMap();
        alocateMap(other.getRows(), other.getCols());

    }

    for (int i = 0; i < getRows(); i++) {
        for (int j = 0; j < getCols(); j++) {
            this->at(i,j) = other.at(i,j);
        }
    }

}

Map::~Map()
{
    deleteMap();
}

Map &Map::operator=(Map &other)
{

    copy(other);
    return *this;

}


Map &Map::tunel(Map &&other)
{
    Cols = other.Cols;
    Rows = other.Rows;
    map = other.map;
    other.map = nullptr;
    other.setCols(0);
    other.setRows(0);
    return *this;
}



void Map::loadFromFile(const char *fileName)
{
    std::ifstream ifst(fileName);

    loadFromFile(ifst);

    ifst.close();
}



void Map::loadFromFile(std::istream &ist)
{
    std::string line;

    int mode = 0;

    while (getline(ist, line)) {
        if (line[0] == '#'){
            continue;
        }
        if (mode == 0) {
            if (line.find("barrier") != line.npos) {
                mode = 1;
            } else if (line.find("goal") != line.npos) {
                mode = 2;
            } else if (line.find("dim") != line.npos) {

                std::stringstream ss(line);

                ss.ignore(50, ' ');

                int r, c;

                ss >> r;
                ss >> c;

                alocateMap(r, c);

            }
        } else if (mode == 1) {
            if (line == ""){
                mode = 0; continue;
            } else {
                int r,c;

                std::stringstream ss(line);
                ss >> r;

                ss >> c;

                at(r,c).setFlag(Field::barier);
            }
        } else if (mode == 2) {
            if (line == ""){
                mode = 0; continue;
            } else {
                int r,c;
                number val;
                std::stringstream ss(line);
                ss >> r;

                ss >> c;

                ss >> val;

                at(r,c).setFlag(Field::goal);
                at(r,c).setValue(val);
            }
        }
    }
}





void Map::deleteMap()
{
    if(getCols() > 0 && getRows() > 0){

        for(int i = 0; i < getRows(); i++){
            delete[] map[i];
        }
        delete[] map;
    }
}




std::ostream &Map::printValues(std::ostream &ost)
{

    for (int i = 0; i < getRows(); i++) {
        for (int j = 0;j < getCols(); j++) {
             ost << std::setw(9) << std::setprecision(3) << at(i,j).getValue();
        }
        ost << std::endl;
    }

    return ost;
}

std::ostream &Map::printDesitions(std::ostream &ost)
{

    for (int i = 0; i < getRows(); i++) {
        for (int j = 0;j < getCols(); j++) {
             ost << std::setw(3);
             switch (at(i,j).getbestChoise()) {

             case Field::north : ost << "^"; break;

             case Field::west : ost << "<"; break;

             case Field::south : ost << "v"; break;

             case Field::east : ost << ">"; break;

             default:
                switch(at(i,j).getFlag()){

                    case Field::barier : ost << "#"; break;

                    case Field::goal : ost << "x"; break;

                default: ost << "err"; break;
                } break;
             }
        }
        ost << std::endl;
    }
    return ost;
}







void Map::alocateMap(const int rows, const int cols)
{
    Cols = cols;
    Rows = rows;
    map = new Field* [rows];
    for (int i = 0; i < rows; i++) {
        map[i] = new Field[cols];
    }
}
