#include "mapview.h"
#include <QGraphicsAnchorLayout>
#include <QGraphicsTextItem>
#include <QGraphicsSceneMouseEvent>
#include <QtDebug>
MapView::MapView(QWidget *parent) : QWidget(parent)
{
    //setMaximumSize(QSize(500,500));


    scene = new myScene;

    view = new QGraphicsView(scene, this);


    lMain = new QVBoxLayout;
    lMain->addWidget(view);

    setLayout(lMain);

    connect(scene, SIGNAL(sceneClicked(QGraphicsSceneMouseEvent*)), this, SLOT(sceneClicked(QGraphicsSceneMouseEvent*)));
}





void MapView::updateView(Map &m)
{

    scene->clear();


//    resize((0.7 + m.getCols()) * 50.0, (m.getRows() + 1) * 50 + 30);

    for (int x=0; x <= m.getCols(); x++)
        scene->addLine(x * 50,0 , x * 50, m.getRows() * 50, QPen(Qt::blue));


    for (int y=0; y <= m.getRows(); y++)
        scene->addLine(0,y * 50 ,m.getCols() * 50, y * 50, QPen(Qt::blue));


    for (int y=0; y < m.getRows(); y++) {
        for (int x=0; x < m.getCols(); x++) {


            int flag = m.at(y,x).getFlag();



            switch(flag){

            case Field::barier :
                scene->addRect(x*50,y*50,50,50, QPen(Qt::gray), QBrush(QColor(50,50,50,128)));
            break;

            case Field::goal :{
                QColor back = m.at(y,x).getValue() > 0 ? QColor(0,255,0,128) : QColor(255,0,0,128);
                scene->addRect(x*50,y*50,50,50, QPen(Qt::gray), QBrush(back));
                if ((viewMode & 1) != 0){
                QGraphicsTextItem *text = scene->addText(QString("%1").arg(m.at(y,x).getValue(),0,'g',3));
                text->setPos(x*50 + 20, y * 50 + 28);
                }
            }
            break;


            case Field::free :
            {
                if ((viewMode & 1) != 0){
                QGraphicsTextItem *text = scene->addText(QString("%1").arg(m.at(y,x).getValue(),0,'g',3));
                text->setPos(x*50 + 25, y * 50 + 25);
                }

            }break;

            }
            if ((viewMode & 2) != 0) {

                if (flag == Field::free){
                    drawArrow(x,y,m.at(y,x).getbestChoise());
                }

            }
        }
    }

}




void MapView::drawArrow(int x, int y, int dir)
{


    QPen pen(QBrush(QColor(0,255,30,128)),1.5);
    switch (dir){
    case Field::north: {
        scene->addLine(x * 50 + 25, y * 50 + 10, x * 50 + 25, y * 50 + 40, pen);
        scene->addLine(x * 50 + 25, y * 50 + 10, x * 50 + 15, y * 50 + 30, pen);
        scene->addLine(x * 50 + 25, y * 50 + 10, x * 50 + 35, y * 50 + 30, pen);
    } break;

    case Field::west: {
        scene->addLine(x * 50 + 10, y * 50 + 25, x * 50 + 40, y * 50 + 25, pen);
        scene->addLine(x * 50 + 10, y * 50 + 25, x * 50 + 30, y * 50 + 15, pen);
        scene->addLine(x * 50 + 10, y * 50 + 25, x * 50 + 30, y * 50 + 35, pen);
    } break;
    case Field::south: {
        scene->addLine(x * 50 + 25, y * 50 + 40, x * 50 + 25, y * 50 + 10, pen);
        scene->addLine(x * 50 + 25, y * 50 + 40, x * 50 + 15, y * 50 + 20, pen);
        scene->addLine(x * 50 + 25, y * 50 + 40, x * 50 + 35, y * 50 + 20, pen);
    } break;
    case Field::east: {
        scene->addLine(x * 50 + 40, y * 50 + 25, x * 50 + 10, y * 50 + 25, pen);
        scene->addLine(x * 50 + 40, y * 50 + 25, x * 50 + 30, y * 50 + 15, pen);
        scene->addLine(x * 50 + 40, y * 50 + 25, x * 50 + 30, y * 50 + 35, pen);
    } break;

    }
}


void MapView::sceneClicked(QGraphicsSceneMouseEvent *me)
{
    qDebug() << "field clicked: " << me->scenePos();
    QPointF mousePos = me->scenePos();
    if (mousePos.x() >= 0 && mousePos.y() >= 0) {
        int i = int(mousePos.x() / 50);
        int j = int(mousePos.y() / 50);
        emit fieldClicked(i, j);
    }

}



